﻿Imports Amazon.S3
Imports Amazon.S3.Model
Imports Amazon.S3.Transfer

Public Class AmazonS3Service

    Private AWSAccessKey As String
    Private AWSSecretKey As String
    Private AWSPrefix As String
    Private AWSClient As AmazonS3
    Private AWSConfig As AmazonS3Config
    Private AWSTransferUtility As TransferUtility
    Private AWSErrorLog As ErrorLogging
    Public Event UploadFileProgress(ByVal sender As Object, ByVal e As UploadProgressArgs)
    Public Event DownloadFileProgress(ByVal sender As Object, ByVal e As WriteObjectProgressArgs)
    Public Event ObjectDeleted(ByVal download As IDownload)

    Public Sub New(ByVal accessKey As String, ByVal secretKey As String, ByVal prefix As String)
        AWSAccessKey = accessKey
        AWSSecretKey = secretKey
        AWSPrefix = prefix

        AWSConfig = New AmazonS3Config
        AWSConfig.WithCommunicationProtocol(Protocol.HTTP)
        'AWSConfig.WithMaxErrorRetry(3)
        'AWSConfig.WithProxyHost("http://192.168.15.1")
        'AWSConfig.WithProxyPort(0)

        AWSClient = Amazon.AWSClientFactory.CreateAmazonS3Client(AWSAccessKey, AWSSecretKey, AWSConfig)

        AWSTransferUtility = New TransferUtility(AWSClient)

        AWSErrorLog = New ErrorLogging
    End Sub

    Public Sub New(ByVal accessKey As String, ByVal secretKey As String, ByVal prefix As String, ByVal proxyHost As String, ByVal proxyPort As UInteger)
        AWSAccessKey = accessKey
        AWSSecretKey = secretKey
        AWSPrefix = prefix

        AWSConfig = New AmazonS3Config
        AWSConfig.WithCommunicationProtocol(Protocol.HTTP)
        'AWSConfig.WithMaxErrorRetry(3)
        AWSConfig.WithProxyHost(proxyHost)
        AWSConfig.WithProxyPort(proxyPort)

        AWSClient = Amazon.AWSClientFactory.CreateAmazonS3Client(AWSAccessKey, AWSSecretKey, AWSConfig)

        AWSTransferUtility = New TransferUtility(AWSClient)

        AWSErrorLog = New ErrorLogging
    End Sub

    Public Function GetBucketList() As Generic.List(Of S3Bucket)
        Dim bucketList As New Generic.List(Of S3Bucket)
        Try
            Dim response As ListBucketsResponse = AWSClient.ListBuckets()
            For Each bucket As S3Bucket In response.Buckets
                bucketList.Add(bucket)
            Next
        Catch amazonS3Exception As AmazonS3Exception
            If Not amazonS3Exception.ErrorCode Is Nothing _
                And (amazonS3Exception.ErrorCode.Equals("InvalidAccessKeyId") _
                     Or amazonS3Exception.ErrorCode.Equals("InvalidSecurity")) Then
                AWSErrorLog.AddLogMessage("Please check the provided AWS Credentials.")
                AWSErrorLog.AddLogMessage("If you haven't signed up for Amazon S3, please visit http://aws.amazon.com/s3")
            Else
                AWSErrorLog.AddLogMessage("An Error, number {0}, occurred when listing buckets with the message '{1}", amazonS3Exception.ErrorCode, amazonS3Exception.Message)
            End If
        Catch ex As Exception
            AWSErrorLog.AddLogMessage(ex.Message)
        Finally
            If AWSErrorLog.HasError Then
                MessageBox.Show(AWSErrorLog.GetLogMessage(), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error)
                AWSErrorLog.ClearLog()
            End If
        End Try

        Return bucketList

    End Function
    ''' <summary>
    '''Amazon S3 bucket names must:
    '''1. Only contain lowercase letters, numbers, periods (.), and dashes (-)
    '''2. Start with a number or letter
    '''3. Be between 3 and 63 characters long
    '''4. Not be in an IP address style (e.g., "192.168.5.4")
    '''5. Not end with a dash
    '''6. Not contain dashes next to periods (e.g., "my-.bucket.com" and "my.-bucket" are invalid)
    ''' </summary>
    ''' <param name="bucketName">Be between 3 and 63 characters long</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function GetObjectList(ByVal bucketName As String) As Generic.List(Of S3Object)
        Dim objectList As New Generic.List(Of S3Object)
        Try
            Dim request As ListObjectsRequest = New ListObjectsRequest()
            request.BucketName = bucketName
            'request.WithPrefix(AWSPrefix)
            Dim response As ListObjectsResponse = AWSClient.ListObjects(request)
            For Each entry As S3Object In response.S3Objects
                objectList.Add(entry)
            Next
        Catch amazonS3Exception As AmazonS3Exception
            If Not amazonS3Exception.ErrorCode Is Nothing _
                And (amazonS3Exception.ErrorCode.Equals("InvalidAccessKeyId") _
                     Or amazonS3Exception.ErrorCode.Equals("InvalidSecurity")) Then
                AWSErrorLog.AddLogMessage("Please check the provided AWS Credentials.")
                AWSErrorLog.AddLogMessage("If you haven't signed up for Amazon S3, please visit http://aws.amazon.com/s3")
            Else
                AWSErrorLog.AddLogMessage("An Error, number {0}, occurred when listing buckets with the message '{1}", amazonS3Exception.ErrorCode, amazonS3Exception.Message)
            End If
        Catch ex As Exception
            AWSErrorLog.AddLogMessage(ex.Message)
        Finally
            If AWSErrorLog.HasError Then
                MessageBox.Show(AWSErrorLog.GetLogMessage(), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error)
                AWSErrorLog.ClearLog()
            End If
        End Try

        Return objectList

    End Function

    Structure TransferFileThreadState
        Dim bucketName As String
        Dim keyName As String
        Dim filePath As String
    End Structure

    Protected Sub UploadProgressEventHandler(ByVal sender As Object, ByVal e As UploadProgressArgs)
        RaiseEvent UploadFileProgress(sender, e)
    End Sub

    Private Sub UploadFileCallBack(ByVal ar As System.IAsyncResult)
        Dim upload As IUpload = ar.AsyncState
        upload.UploadStatus = "Completed"
    End Sub

    Public Sub UploadFile(ByVal upload As IUpload)
        Try
            Dim bucketName As String, keyName As String, filePath As String
            bucketName = upload.BucketName
            keyName = upload.KeyName
            filePath = upload.UploadFilePath

            'Using Transfer Utility
            Dim request As TransferUtilityUploadRequest = New TransferUtilityUploadRequest()
            request.WithBucketName(bucketName) _
            .WithKey(AWSPrefix & "/" & keyName) _
            .WithFilePath(filePath) _
            .WithSubscriber(AddressOf UploadProgressEventHandler) _
            .WithPartSize(1048576) _
            .WithStorageClass(S3StorageClass.ReducedRedundancy)

            'request.WithTimeout(30 * 60 * 1000)

            AWSTransferUtility.BeginUpload(request, New System.AsyncCallback(AddressOf UploadFileCallBack), upload)

        Catch amazonS3Exception As AmazonS3Exception
            If Not amazonS3Exception.ErrorCode Is Nothing _
                And (amazonS3Exception.ErrorCode.Equals("InvalidAccessKeyId") _
                     Or amazonS3Exception.ErrorCode.Equals("InvalidSecurity")) Then
                AWSErrorLog.AddLogMessage("Please check the provided AWS Credentials.")
                AWSErrorLog.AddLogMessage("If you haven't signed up for Amazon S3, please visit http://aws.amazon.com/s3")
            Else
                AWSErrorLog.AddLogMessage("An Error, number {0}, occurred when listing buckets with the message '{1}", amazonS3Exception.ErrorCode, amazonS3Exception.Message)
            End If
        Catch ex As Exception
            AWSErrorLog.AddLogMessage(ex.Message)
        Finally
            If AWSErrorLog.HasError Then
                MessageBox.Show(AWSErrorLog.GetLogMessage(), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error)
                AWSErrorLog.ClearLog()
            End If

        End Try

    End Sub

    Protected Sub DownloadProgressEventHandler(ByVal sender As Object, ByVal e As WriteObjectProgressArgs)
        RaiseEvent DownloadFileProgress(sender, e)
    End Sub

    Private Sub DownloadFileCallBack(ByVal ar As System.IAsyncResult)
        Dim download As IDownload = ar.AsyncState
        download.DownloadStatus = "Completed"
    End Sub

    Public Sub DownloadFile(ByVal download As IDownload)
        Try
            Dim bucketName As String, keyName As String, filePath As String

            bucketName = download.BucketName
            keyName = download.DownloadKeyName
            filePath = download.DowloadFilePath & "\" & download.DownloadFileName

            'Using Transfer Utility
            Dim request As TransferUtilityDownloadRequest = New TransferUtilityDownloadRequest()
            request.WithBucketName(bucketName) _
            .WithKey(keyName) _
            .WithFilePath(filePath) _
            .WithSubscriber(AddressOf DownloadProgressEventHandler)
            AWSTransferUtility.BeginDownload(request, New System.AsyncCallback(AddressOf DownloadFileCallBack), download)

        Catch amazonS3Exception As AmazonS3Exception
            If Not amazonS3Exception.ErrorCode Is Nothing _
                And (amazonS3Exception.ErrorCode.Equals("InvalidAccessKeyId") _
                     Or amazonS3Exception.ErrorCode.Equals("InvalidSecurity")) Then
                AWSErrorLog.AddLogMessage("Please check the provided AWS Credentials.")
                AWSErrorLog.AddLogMessage("If you haven't signed up for Amazon S3, please visit http://aws.amazon.com/s3")
            Else
                AWSErrorLog.AddLogMessage("An Error, number {0}, occurred when listing buckets with the message '{1}", amazonS3Exception.ErrorCode, amazonS3Exception.Message)
            End If
        Catch ex As Exception
            AWSErrorLog.AddLogMessage(ex.Message)
        Finally
            If AWSErrorLog.HasError Then
                MessageBox.Show(AWSErrorLog.GetLogMessage(), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error)
                AWSErrorLog.ClearLog()
            End If

        End Try
    End Sub

    Public Sub DeleteObject(ByVal download As IDownload)
        Try
            Dim bucketName As String, keyName As String
            bucketName = download.BucketName
            keyName = download.DownloadKeyName
            Dim request As DeleteObjectRequest = New DeleteObjectRequest()
            request.WithBucketName(bucketName)
            request.WithKey(keyName)

            Dim response As S3Response = AWSClient.DeleteObject(request)
            response.Dispose()
        Catch amazonS3Exception As AmazonS3Exception
            If Not amazonS3Exception.ErrorCode Is Nothing _
                And (amazonS3Exception.ErrorCode.Equals("InvalidAccessKeyId") _
                     Or amazonS3Exception.ErrorCode.Equals("InvalidSecurity")) Then
                AWSErrorLog.AddLogMessage("Please check the provided AWS Credentials.")
                AWSErrorLog.AddLogMessage("If you haven't signed up for Amazon S3, please visit http://aws.amazon.com/s3")
            Else
                AWSErrorLog.AddLogMessage("An Error, number {0}, occurred when listing buckets with the message '{1}", amazonS3Exception.ErrorCode, amazonS3Exception.Message)
            End If
        Catch ex As Exception
            AWSErrorLog.AddLogMessage(ex.Message)
        Finally
            If AWSErrorLog.HasError Then
                MessageBox.Show(AWSErrorLog.GetLogMessage(), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error)
                AWSErrorLog.ClearLog()
            Else
                RaiseEvent ObjectDeleted(download)
            End If

        End Try
    End Sub

    Public Sub CreateBucket(ByVal bucket As IBucket)
        Try
            If bucket.NewBucketName = "" Then Throw New Exception("Bucket name is missing, please enter bucket name.")
            If bucket.NewBucketLocationExist = False Then Throw New Exception("Bucket region is missing, please select bucket region.")
            Dim request As PutBucketRequest = New PutBucketRequest()
            request.BucketName = bucket.NewBucketName
            request.BucketRegion = bucket.NewBucketLocation
            AWSClient.PutBucket(request)
            MessageBox.Show("Bucket created successfully.")
        Catch amazonS3Exception As AmazonS3Exception
            If Not amazonS3Exception.ErrorCode Is Nothing _
                And (amazonS3Exception.ErrorCode.Equals("InvalidAccessKeyId") _
                     Or amazonS3Exception.ErrorCode.Equals("InvalidSecurity")) Then
                AWSErrorLog.AddLogMessage("Please check the provided AWS Credentials.")
                AWSErrorLog.AddLogMessage("If you haven't signed up for Amazon S3, please visit http://aws.amazon.com/s3")
            Else
                AWSErrorLog.AddLogMessage("An Error, number {0}, occurred when listing buckets with the message '{1}", amazonS3Exception.ErrorCode, amazonS3Exception.Message)
            End If
        Catch ex As Exception
            AWSErrorLog.AddLogMessage(ex.Message)
        Finally
            If AWSErrorLog.HasError Then
                MessageBox.Show(AWSErrorLog.GetLogMessage(), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error)
                AWSErrorLog.ClearLog()
            End If

        End Try
    End Sub

    Public Sub DeleteBucket(ByVal bucket As IBucket)
        Try
            If bucket.BucketName = "" Then Throw New Exception("Bucket name is missing, please select bucket name.")
            Dim request As DeleteBucketRequest = New DeleteBucketRequest()
            request.BucketName = bucket.BucketName
            AWSClient.DeleteBucket(request)
            MessageBox.Show("Bucket deleted successfully.")
        Catch amazonS3Exception As AmazonS3Exception
            If Not amazonS3Exception.ErrorCode Is Nothing _
                And (amazonS3Exception.ErrorCode.Equals("InvalidAccessKeyId") _
                     Or amazonS3Exception.ErrorCode.Equals("InvalidSecurity")) Then
                AWSErrorLog.AddLogMessage("Please check the provided AWS Credentials.")
                AWSErrorLog.AddLogMessage("If you haven't signed up for Amazon S3, please visit http://aws.amazon.com/s3")
            Else
                AWSErrorLog.AddLogMessage("An Error, number {0}, occurred when listing buckets with the message '{1}", amazonS3Exception.ErrorCode, amazonS3Exception.Message)
            End If
        Catch ex As Exception
            AWSErrorLog.AddLogMessage(ex.Message)
        Finally
            If AWSErrorLog.HasError Then
                MessageBox.Show(AWSErrorLog.GetLogMessage(), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error)
                AWSErrorLog.ClearLog()
            End If

        End Try
    End Sub
End Class
